/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UKUIVERSION4TRAYICON_H
#define UKUIVERSION4TRAYICON_H

#include <memory>
#include <QAction>
#include <QObject>

#include "ISoundSystemTrayIcon.h"

class IMainWindow;
class UkuiVersion4TrayIcon : public QObject, public ISoundSystemTrayIcon
{
    Q_OBJECT
public:
    UkuiVersion4TrayIcon(UkmediaTrayIcon*, IMainWindow*);
    ~UkuiVersion4TrayIcon() = default;

public:
    virtual void initTray() override;

    virtual void initSlot() override;

    virtual void setVisible(bool) override;

    virtual void setIcon(const QIcon&) override;

private slots:
    void soundPreferenceActionTriggered();

    void activatedSystemTrayIconSlot(QSystemTrayIcon::ActivationReason);

private:
    IMainWindow* m_pMainWid;
};

#endif // UKUIVERSION4TRAYICON_H
