/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APPMANAGERSLIDERITEM_H
#define APPMANAGERSLIDERITEM_H

#include <QFrame>

#include "IVolumeSliderItem.h"

class AppManagerSliderItem : public QFrame, public IVolumeSliderItem
{
    Q_OBJECT
public:
    AppManagerSliderItem(std::shared_ptr<IVolumeSlider>, QLabel*, QLabel*, QPushButton*, VolumeSliderType, QWidget* = nullptr);
    ~AppManagerSliderItem() = default;

public:
    virtual void initUi() override;

    virtual void initSlots() override;

    virtual QWidget* getWidget() override;

    virtual void setValue(int) override;

    virtual void setEnabled(bool) override;

    virtual QPushButton* getMuteBtn() const override;

    virtual void setMute(bool) override;

private Q_SLOTS:
    void valueChangedSlots(int);

private:
    void setPercent(int);

    void setIcon(const QIcon&);

    void updateIcon(int);

private:
    VolumeSliderType m_type = VolumeSliderType::VOLUME_SLIDER_INPUT;
    QLabel* m_pPercentLabel = nullptr;
    QPushButton* m_pMuteBtn = nullptr;
    bool m_muted = false;
};

#endif // APPMANAGERSLIDERITEM_H
