/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "SingleApplication.h"
#include <iostream>

SingleApplication::SingleApplication(const std::string& name)
    : m_name(name)
{
    m_lockFile = open(("/tmp/" + m_name + "-" + std::to_string(getuid())).c_str(), O_CREAT | O_RDWR, 0666);
    if (m_lockFile != -1 && flock(m_lockFile, LOCK_EX | LOCK_NB) == -1) {
        close(m_lockFile);
        m_lockFile = -1;
    }
}

SingleApplication::~SingleApplication()
{
    if (m_lockFile != -1) {
        flock(m_lockFile, LOCK_UN);
        close(m_lockFile);
    }
}

bool SingleApplication::isAlreadyRuning()
{
    return m_lockFile == -1;
}
