/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ISYSTEMVOLUMEWIDGET_H
#define ISYSTEMVOLUMEWIDGET_H

#include <QWidget>

#include "IDevicePortSelectItem.h"

class ISystemVolumeWidget
{
public:
    virtual void initUi() = 0;

    virtual void initSlots() = 0;

    virtual void showWidget() = 0;

    virtual QWidget* getWidget() = 0;

    virtual void setValue(int) = 0;

    virtual void addDevice(const QString&, const QVariantList&) = 0;

    virtual void removeDevice(const QString&, const QString&) = 0;

    virtual void addDevicePortItem(std::shared_ptr<IDevicePortItem>) = 0;

    virtual void removeDevicePortItem(const QString&, const QVariantList&) = 0;

    virtual void setCurrentIndex(int) = 0;

protected:
    std::shared_ptr<IDevicePortSelectItem> m_pListItem = nullptr;
};

#endif // ISYSTEMVOLUMEWIDGET_H
