/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SOUNDTHEMEPLAYERSETTING_H
#define SOUNDTHEMEPLAYERSETTING_H

#include <glib.h>
#include <gio/gio.h>
#include "ISetting.h"

namespace UkuiAudioFramwork
{
#define SOUND_THEME_PLAYER_SCHEMA  "org.ukui.media.sound"
#define SOUND_THEME_PLAYER_PATH    "/org/ukui/sound/keybindings/"
#define SOUND_FILENAME_KEY         "filename"
#define SOUND_NAME_KEY             "name"

class SoundThemePlayerSetting : public ISetting
{
public:
    SoundThemePlayerSetting();
    ~SoundThemePlayerSetting();

public:
    void initConnect();

    virtual void initSettings() override;

    virtual GVariant* getValue(const char*) const override;

    virtual void setValue(const char*, GVariant*) override;

    virtual bool isContainKeys(const char*) const override;

private:
    static void onSettingsChanged(GSettings*, const gchar*);

private:
    GSettings* m_pSettings = nullptr;
    GSettingsSchema* m_pSchema = nullptr;
    GSettingsSchemaSource* m_pSource = nullptr;
};
}
#endif // SOUNDTHEMEPLAYERSETTING_H
