/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APPMANAGERSELECTCOMBOXITEM_H
#define APPMANAGERSELECTCOMBOXITEM_H

#include <QHBoxLayout>

#include "ISelectComboxItem.h"

class AppManagerSelectComboxItem : public QFrame, public ISelectComboxItem
{
    Q_OBJECT
public:
    AppManagerSelectComboxItem(std::shared_ptr<ISelectBox>, QLabel*, QWidget* = nullptr);
    ~AppManagerSelectComboxItem() = default;

public:
    virtual void initUi() override;

    virtual void initSlots() override;

    virtual void setEnabled(bool) override;

    virtual QWidget* getWidget() override;
};

class AppSelectComboxWithDescItem : public QFrame, public ISelectComboxItem
{   
    Q_OBJECT
public:
    AppSelectComboxWithDescItem(std::shared_ptr<ISelectBox>, QLabel*, QLabel*, QWidget* = nullptr);
    ~AppSelectComboxWithDescItem() = default;

public:
    virtual void initUi() override;

    virtual void initSlots() override;

    virtual void setEnabled(bool) override;

    virtual QWidget* getWidget() override;

private:
    QLabel* m_pHintLabel {};
    QLabel* m_pHintIconLabel {};
    QVBoxLayout* m_pVLayout {};
    QHBoxLayout* m_pHintHLayout {};
};

#endif // APPMANAGERSELECTCOMBOXITEM_H
