/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TRAYCLIENTMANAGER_H
#define TRAYCLIENTMANAGER_H

#include <memory>
#include <unordered_map>
#include <QIcon>
#include <QVariant>

#include "ITrayVolumeSliderItem.h"
#include "IMainWindow.h"
#include "DBusClient.h"

class TrayClientManager
{
public:
    TrayClientManager(std::unordered_map<int, std::shared_ptr<ITrayVolumeSliderItem>>&,
                      std::unordered_map<int, std::shared_ptr<ITrayVolumeSliderItem>>&,
                      IMainWindow*);
    virtual ~TrayClientManager() = default;

public:
    void setValue(int, int, int);

    void setIcon(int, int, const QIcon&);

    void setCurrentIndex(int idx);

    void addDevicePortItem(int, const ClientDeviceInfo&);

    void removeDevicePortItem(const QString&, const QString&);

    void addAppItem(int, const QString&, const QString&);

    void removeAppItem(int);

    void updateDescription(const QString&, const QString&);

private:
    std::unordered_map<int, std::shared_ptr<ITrayVolumeSliderItem>>& m_volumeSliderItemMap;
    std::unordered_map<int, std::shared_ptr<ITrayVolumeSliderItem>>& m_appVolumeSliderItemMap;
    IMainWindow* m_pMainWid = nullptr;
};


#endif // TRAYCLIENTMANAGER_H
