/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CLIENTMETHOD_H
#define CLIENTMETHOD_H

#include <memory>
#include "DBusClient.h"
#include "ClientManager.h"

class ClientMethod
{
public:
    static ClientMethod& getInstance();

    void setManager(std::shared_ptr<ClientManager>);

    void setValue(int, int);

    void setMute(int, bool);

    void setEnabled(int, bool);

    void setBalanceEnabled(bool);

    void setRange(int, int, int);

    void setChecked(int, bool);

    void setCurrentIndex(int, int);

    int findData(int, const QVariant&, int = Qt::UserRole) const;

    int findText(int, const QString&) const;

    void addItem(int, const QString&, const QVariant& = QVariant());

    void removeItem(int, int);

    void clear(int);

private:
    ClientMethod() = default;
    ClientMethod(const ClientMethod&) = delete;
    ClientMethod(ClientMethod&&) = delete;
    ClientMethod operator=(const ClientMethod&) = delete;
    ClientMethod operator=(ClientMethod&&) = delete;
    virtual ~ClientMethod() = default;

private:
    std::shared_ptr<ClientManager> m_pClientManager{nullptr};
};

#endif // CLIENTMETHOD_H
