/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UKUI2MINIWIDGET_H
#define UKUI2MINIWIDGET_H

#include <QWidget>
#include <QLabel>

#include "ITrayVolumeSliderItem.h"

class Ukui2MiniWidget : public QWidget
{
public:
    Ukui2MiniWidget(std::unordered_map<int, std::shared_ptr<ITrayVolumeSliderItem>>&, QPushButton*, QWidget*);
    ~Ukui2MiniWidget() = default;

public:
    void setDescription(const QString&);

private:
    void initUi();

    void initSlots();

private:
    QLabel* m_pLabel = nullptr;
    QPushButton* m_pIconBtn = nullptr;
    QPushButton* m_pSwitchModuleBtn = nullptr;
    std::unordered_map<int, std::shared_ptr<ITrayVolumeSliderItem>>& m_sysVolumeSliderItemMap;
};

#endif // UKUI2MINIWIDGET_H
