/*
* Copyright (C) 2024, KylinSoft Co., Ltd.
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*
* Authors: zhouyong <zhouyong01@kylinos.cn>
*
*/

#include "common.h"
#include <QCoreApplication>
#include <QDebug>

Common& Common::getInstance()
{
    static Common instance;
    return instance;
}

QVariant Common::getThemePropertyByName(QString name)
{
    qDebug() << __func__ << name << qApp->property(name.toStdString().c_str());
    return qApp->property(name.toStdString().c_str());
}
