/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TITLELABELITEM_H
#define TITLELABELITEM_H

#include <QLabel>

/**
* @brief 包含TitleLabel子项的类型
*/
enum class TitleLabelType
{
    TITLE_LABEL_OUTPUT = 0,         // 输出标题
    TITLE_LABEL_INPUT,              // 输入标题
    TITLE_LABEL_SOUNDEFFECT,        // 系统音效标题
    TITLE_LABEL_ADVANCESETTINGS     // 高级设置标题
};

class TitleLabelItem : public QWidget
{
public:
    TitleLabelItem(QLabel*, QWidget* = nullptr);
    virtual ~TitleLabelItem() = default;

public:
    void initUi();

    QWidget* getWidget();

private:
    QLabel* m_pLabel;
};

#endif // TITLELABELITEM_H
