/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MULTIAUDIOCOMBINEJSON_H
#define MULTIAUDIOCOMBINEJSON_H

#include <vector>
#include "IJson.h"

namespace UkuiAudioFramwork
{
#define MULTI_AUDIO_COMBINE_JSON  "/.config/ukui-volume-control/multi_audio_combine.json"

typedef struct MultiAudioCombineInfo {
    int type;       // 0 输入  1 输出
    bool status;
    std::vector<std::string> deviceList;
} MultiAudioCombineInfo;

class MultiAudioCombineJson : public IJson<MultiAudioCombineInfo>
{
public:
    MultiAudioCombineJson();
    ~MultiAudioCombineJson() = default;

public:
    virtual bool isContain(const MultiAudioCombineInfo&) const override;

    virtual void insert(const MultiAudioCombineInfo&) override;

//    template<typename T>
//    std::optional<T> getValue(const std::string&, const std::string&, const std::string&) const;

    std::optional<bool> getValue(const std::string&, const std::string&) const;

    void setValue(const std::string&, const std::string&, bool);

private:
    void initJson();

};
}
#endif // MULTIAUDIOCOMBINEJSON_H
