/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IDETAILSETTINGSITEM_H
#define IDETAILSETTINGSITEM_H

#include <memory>
#include <QWidget>
#include <QLabel>

/**
* @brief 包含DetailSettings子项的类型
*/
enum class DetailSettingType
{
    DETAIL_SETTING_DEVMANAGER = 0,  // 设备管理子项
    DETAIL_SETTING_APPMANAGER,      // 应用声音子项
};

class IDetailSettings {
public:
    virtual QWidget* getWidget() = 0;
};


class IDetailSettingsItem
{
public:
    IDetailSettingsItem(std::shared_ptr<IDetailSettings>, QLabel*);
    virtual ~IDetailSettingsItem() = default;

public:
    virtual void initUi() = 0;

    virtual void initSlots() = 0;

    virtual QWidget* getWidget() = 0;

    std::shared_ptr<IDetailSettings> getDetailSettings() const;

protected:
    std::shared_ptr<IDetailSettings> m_pDetailSettings = nullptr;
    QLabel* m_pDisplayLabel = nullptr;
};

#endif // IDETAILSETTINGSITEM_H
