/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ConcreteStrategy.h"

IAudioMainWindow* Ukui2ConcreteStrategy::createMainWindow(std::unordered_map<TitleLabelType, TitleLabelItem*>& titleLabelMap,
                                                          std::unordered_map<SwitchButtonType, ISwitchButtonItem*>& switchBtnMap,
                                                          std::unordered_map<VolumeSliderType, IVolumeSliderItem*>& volumeSliderMap,
                                                          std::unordered_map<SelectComboxType, ISelectComboxItem*>& selectComboxMap,
                                                          std::unordered_map<DetailSettingType, IDetailSettingsItem*>& detailSettingsMap,
                                                          std::list<AppManagerItem>& appManagerList,
                                                          QWidget* parent)
{
    return new Ukui4MainWidget(titleLabelMap, switchBtnMap, volumeSliderMap, selectComboxMap, detailSettingsMap, appManagerList, parent);
}

IAudioMainWindow* Ukui4ConcreteStrategy::createMainWindow(std::unordered_map<TitleLabelType, TitleLabelItem*>& titleLabelMap,
                                                          std::unordered_map<SwitchButtonType, ISwitchButtonItem*>& switchBtnMap,
                                                          std::unordered_map<VolumeSliderType, IVolumeSliderItem*>& volumeSliderMap,
                                                          std::unordered_map<SelectComboxType, ISelectComboxItem*>& selectComboxMap,
                                                          std::unordered_map<DetailSettingType, IDetailSettingsItem*>& detailSettingsMap,
                                                          std::list<AppManagerItem>& appManagerList,
                                                          QWidget* parent)
{
    return new Ukui4MainWidget(titleLabelMap, switchBtnMap, volumeSliderMap, selectComboxMap, detailSettingsMap, appManagerList, parent);
}

IAudioMainWindow* Ukui5ConcreteStrategy::createMainWindow(std::unordered_map<TitleLabelType, TitleLabelItem*>& titleLabelMap,
                                                          std::unordered_map<SwitchButtonType, ISwitchButtonItem*>& switchBtnMap,
                                                          std::unordered_map<VolumeSliderType, IVolumeSliderItem*>& volumeSliderMap,
                                                          std::unordered_map<SelectComboxType, ISelectComboxItem*>& selectComboxMap,
                                                          std::unordered_map<DetailSettingType, IDetailSettingsItem*>& detailSettingsMap,
                                                          std::list<AppManagerItem>& appManagerList,
                                                          QWidget* parent)
{
    return new Ukui5MainWidget(titleLabelMap, switchBtnMap, volumeSliderMap, selectComboxMap, detailSettingsMap, appManagerList, parent);
}
