/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RESTOREVOLUMEJSON_H
#define RESTOREVOLUMEJSON_H

#include <variant>

#include "IJson.h"

namespace UkuiAudioFramwork
{
#define RESTORE_VOLUME_JSON  "/.config/ukui-volume-control/restore-volume.json"

struct  RestoreVolumeInfo
{
    std::string cardName;
    std::string portName;
    uint32_t volume;
    uint32_t priority;
    double balance;
    bool enable;
    bool muted;
};

using MemberVarPtr = std::variant<
    uint32_t RestoreVolumeInfo::*,
    double RestoreVolumeInfo::*,
    bool RestoreVolumeInfo::*
>;

using FieldMap = std::vector<std::pair<const char*, MemberVarPtr>>;

class RestoreVolumeJson : public IJson<RestoreVolumeInfo>
{
public:
    RestoreVolumeJson();
    ~RestoreVolumeJson() = default;

public:
    virtual bool isContain(const RestoreVolumeInfo&) const override;

    virtual void insert(const RestoreVolumeInfo&) override;

//    template<typename T>
//    std::optional<T> getValue(const std::string&, const std::string&, const std::string&) const;

    std::optional<bool> getValue(const std::string&) const;

    bool getStatus(const std::string&, const std::string&, const std::string&) const;

    void setValue(const std::string&, bool);

private:
    void initJson();

private:
    const FieldMap m_filedMap = {
        {"volume",    &RestoreVolumeInfo::volume},
        {"priority",  &RestoreVolumeInfo::priority},
        {"balance",   &RestoreVolumeInfo::balance},
        {"enable",    &RestoreVolumeInfo::enable},
        {"mute",      &RestoreVolumeInfo::muted}
    };
};
}
#endif // RESTOREVOLUMEJSON_H
