/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CustomAudioVolumeJson.h"

#include <iostream>
#include <fstream>
#include <filesystem>
#include <iomanip>

namespace UkuiAudioFramwork
{
CustomAudioVolumeJson::CustomAudioVolumeJson()
{
    m_fileName = getenv("HOME");
    m_fileName += CUSTOM_AUDIO_VOLUME_JSON;
    loadJson();
}

bool CustomAudioVolumeJson::isContain(const AudioVolumeInfo& info) const
{
    const auto& card = m_json.find(info.cardName);
    if (card != m_json.end()) {
        const auto& port = (*card).find(info.portName);
        return port != (*card).end();
    }
    return false;
}

void CustomAudioVolumeJson::insert(const AudioVolumeInfo& info)
{
    nlohmann::json& ports = m_json[info.cardName];
    if (ports.find(info.portName) == ports.end()) {
        ports[info.portName] = {
            {"name", info.portName},
            {"volume", info.volume}
        };
    }

    writeToJson();
}
}
