/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MEDIACONTROLER_H
#define MEDIACONTROLER_H

#include <glib.h>
#include <gio/gio.h>
#include <vector>
#include <string>

namespace UkuiAudioFramwork
{
class MediaControler
{
public:
    static MediaControler& getInstance();

public:
    void pauseAllPlayers();

private:
    std::vector<std::string> getMprisPlayers(GDBusConnection*) const;

private:
    MediaControler() = default;
    MediaControler(const MediaControler&) = delete;
    MediaControler(MediaControler&&) = delete;
    MediaControler operator=(const MediaControler&) = delete;
    MediaControler operator=(MediaControler&&) = delete;
    virtual ~MediaControler() = default;
};
}
#endif // MEDIACONTROLER_H
