/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DEVICEMANAGERMODULE_H
#define DEVICEMANAGERMODULE_H

#include "Util.h"
#include "AudioManager.h"
#include "RestoreVolumeJson.h"

namespace UkuiAudioFramwork
{
class AudioManager;

class DeviceManagerModule
{
public:
    static DeviceManagerModule& getInstance();

    void setManager(std::shared_ptr<AudioManager>);

    void setJson(std::shared_ptr<RestoreVolumeJson>);

    void setEnabled(const std::string&, const std::string&, bool);

    bool isEnabled(const std::string&, const std::string&) const;

private:
    DeviceManagerModule() = default;
    DeviceManagerModule(const DeviceManagerModule&) = delete;
    DeviceManagerModule(DeviceManagerModule&&) = delete;
    DeviceManagerModule operator=(const DeviceManagerModule&) = delete;
    DeviceManagerModule operator=(DeviceManagerModule&&) = delete;
    virtual ~DeviceManagerModule() = default;

private:
    std::shared_ptr<RestoreVolumeJson> m_pJson {};
    std::shared_ptr<AudioManager> m_pAudioManager {};
};
}
#endif // DEVICEMANAGERMODULE_H
