/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ITRAYVOLUMESLIDERITEM_H
#define ITRAYVOLUMESLIDERITEM_H

#include <memory>
#include <QLabel>
#include <QPushButton>

class ITrayVolumeSlider {
public:
    virtual void setRang(int, int) = 0;
    virtual void setValue(int) = 0;
    virtual QWidget* getWidget() = 0;
};

class ITrayVolumeSliderItem {
public:
    ITrayVolumeSliderItem(std::shared_ptr<ITrayVolumeSlider>, QPushButton*);
    virtual ~ITrayVolumeSliderItem() {}

public:
    virtual void initUi() = 0;

    virtual void initSlots() = 0;

    virtual void setIcon(const QIcon&) = 0;

    virtual QWidget* getWidget() = 0;

    virtual void setValue(int) = 0;

    virtual QPushButton* getMuteButton() = 0;

    std::shared_ptr<ITrayVolumeSlider> getSlider() const;

    QPushButton* getIconButton() const;

protected:
    std::shared_ptr<ITrayVolumeSlider> m_pVolumeSlider = nullptr;
    QPushButton* m_pIconBtn = nullptr;
};

#endif // ITRAYVOLUMESLIDERITEM_H
