/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UKCC5DEVICEPORTITEM_H
#define UKCC5DEVICEPORTITEM_H

#include <QWidget>
#include <QLabel>
#include <QPushButton>

#include "IDevicePortSelectItem.h"

class Ukcc5DevicePortItem : public QWidget, public IDevicePortItem
{
public:
    Ukcc5DevicePortItem(QLabel*, QLabel*, const QString&, const QString&, QWidget* = nullptr);
    ~Ukcc5DevicePortItem() = default;

public:
    void initUi();

    virtual QWidget* getWidget() override;

    virtual bool isChecked() override;

    virtual void setChecked(bool) override;

    virtual void setIcon(const QIcon&) override;

private:
    QLabel* m_pDisplayLabel = nullptr;
    QLabel* m_pIconLabel = nullptr;
    bool m_bIsChecked = false;
};
#endif // UKCC5DEVICEPORTITEM_H
