/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef INITVOLUMEMODULE_H
#define INITVOLUMEMODULE_H

#include "CustomAudioVolumeJson.h"

namespace UkuiAudioFramwork
{
class InitVolumeModule
{
public:
    static InitVolumeModule& getInstance();

    bool isNeedInitVolume(const std::string&, const std::string&);

    void insert(const std::string&, const std::string&, uint32_t);

private:
    InitVolumeModule() = default;
    InitVolumeModule(const InitVolumeModule&) = delete;
    InitVolumeModule(InitVolumeModule&&) = delete;
    InitVolumeModule operator=(const InitVolumeModule&) = delete;
    InitVolumeModule operator=(InitVolumeModule&&) = delete;
    virtual ~InitVolumeModule() = default;

private:
    CustomAudioVolumeJson m_json;
};
}
#endif // INITVOLUMEMODULE_H
