/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IDEVICEPORTITEM_H
#define IDEVICEPORTITEM_H

#include <memory>
#include <QWidget>
#include <QVariant>

class IDevicePortItem
{
public:
    IDevicePortItem(const QString& cardName, const QString& portName) : m_devName(cardName), m_portName(portName) {}
    virtual ~IDevicePortItem() {}

public:
    virtual QWidget* getWidget() = 0;

    virtual bool isChecked() = 0;

    virtual void setChecked(bool) = 0;

    virtual void setIcon(const QIcon&) = 0;

    QString getCardName() const {return m_devName;}

    QString getPortName() const {return m_portName;}

protected:
    QString m_devName;
    QString m_portName;
};

class IDevicePortSelectItem {
public:
    virtual void initSlots() = 0;

    virtual void setCurrentIndex(int) = 0;

    virtual void insertItem(std::shared_ptr<IDevicePortItem>) = 0;

    virtual void removeItem(const QString&, const QString&) = 0;

    virtual QVariant currentData(int = Qt::UserRole) const = 0;

    virtual int count() const = 0;

    virtual QWidget* getWidget() = 0;
};

#endif // IDEVICEPORTITEM_H
