/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UKUI5DETAILSETTINGSITEM_H
#define UKUI5DETAILSETTINGSITEM_H

#include <QPushButton>

#include "IDetailSettingsItem.h"

class Ukui5DetailSettings : public IDetailSettings
{
public:
    Ukui5DetailSettings(QPushButton*, const QString& = "");
    ~Ukui5DetailSettings() = default;

public:
    virtual QWidget* getWidget() override;

private:
    QPushButton* m_pSettingBtn;
};

class Ukui5DetailSettingsItem : public QFrame, public IDetailSettingsItem
{
public:
    Ukui5DetailSettingsItem(std::shared_ptr<IDetailSettings>, QLabel*, QWidget* = nullptr);

public:
    virtual void initUi() override;

    virtual void initSlots() override;

    virtual QWidget* getWidget() override;
};

#endif // UKUI5DETAILSETTINGSITEM_H
