/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BASETYPE_H
#define BASETYPE_H

#include <list>
#include <memory>
#include <unordered_map>
#include <string>
#include <cstdint>

#include "simple_logger/Formatter.h"

namespace UkuiAudioFramwork
{
enum class Module
{
    Service = 0,
    Tray,
    ControlCenter
};

//
enum class BackendType
{
    PULSEAUDIO = 0,
    PIPEWIRE,
    ALSA,
    JACK,
    UNKNOWN
};

enum class VolumeType
{
    BASE_VOLUME = 0,
    BALANCE_VOLUME
};

enum class NodeType
{
    INPUT = 0,
    OUTPUT,
    SINK_INPUT,
    SOURCE_OUTPUT
};

enum class PaDataType
{
    CARD = 0,
    SINK,
    SOURCE,
    SINK_INPUT,
    SOURCE_OUTPUT,
    MODULE
};

typedef struct NodeInfo
{
    uint32_t index;
    uint8_t channels;
} NodeInfo;
}

#endif // BASETYPE_H
