/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2025 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TRAYCLIENTMETHOD_H
#define TRAYCLIENTMETHOD_H

#include <memory>
#include "DBusClient.h"
#include "TrayClientManager.h"

class TrayClientMethod
{
public:
    static TrayClientMethod& getInstance();

public:
    void setManager(std::shared_ptr<TrayClientManager>);

    void setValue(int, int, int) ;

    void setIcon(int, const QIcon&);

    void setChecked(int, bool);

    void setCurrentIndex(int);

    void addDeviceItem(int, const ClientDeviceInfo&);

    void removeDeviceItem(const QString&, const QString&);

    void addAppItem(int, const QString&, const QString&);

    void removeAppItem(int);

    void updateDescription(const QString&, const QString&);

private:
    TrayClientMethod() = default;
    TrayClientMethod(const TrayClientMethod&) = delete;
    TrayClientMethod(TrayClientMethod&&) = delete;
    TrayClientMethod operator=(const TrayClientMethod&) = delete;
    TrayClientMethod operator=(TrayClientMethod&&) = delete;
    virtual ~TrayClientMethod() = default;

private:
    std::shared_ptr<TrayClientManager> m_pClientManager{nullptr};
};

#endif // TRAYCLIENTMETHOD_H
